<?php
/**
 * @package    Hikashop shipping plugin providing extra features
 * @author     https://www.brainforge.co.uk
 * @copyright  (C) 2024 Jonathan Brain. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

namespace Brainforgeuk\Plugin\HikashopShipping\BfExtraFeatures\Helper;

use Joomla\CMS\Factory;

\defined('_JEXEC') or die('Restricted access');

class DatabaseHelper
{
	/*
	 */
	public static function quote($value, $escape = true)
	{
		if (is_array($value))
		{
			foreach($value as &$data)
			{
				$data = self::quote($data, $escape);
			}
			unset($data);

			return implode(',', $value);
		}

		$db = Factory::getContainer()->get('DatabaseDriver');
		return $db->quote($value, $escape);
	}

	/*
	 */
	public static function quoteName($name, $as=null)
	{
		if (is_array($name))
		{
			foreach($name as &$item)
			{
				$item = self::quoteName($item);
			}
			unset($item);

			return implode(',', $name);
		}

		$db = Factory::getContainer()->get('DatabaseDriver');
		return $db->quoteName($name, $as);
	}

	/*
	 */
	public static function quoteOrNull($value, $escape = true)
	{
		if (is_array($value))
		{
			foreach($value as &$data)
			{
				$data = self::quoteOrNull($data, $escape);
			}
			unset($data);

			return implode(',', $value);
		}

		if ($value == '' || $value === null)
		{
			return 'NULL';
		}

		return self::quote($value, $escape);
	}

	/*
	 */
	public static function escape($text, $extra=false)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');
		return $db->escape($text, $extra);
	}

	/*
	 */
	public static function loadColumn($query, $offset = 0)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');

		$db->setQuery($query);
		return $db->loadColumn($offset);
	}

	/*
	 */
	public static function loadResult($query)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');

		$db->setQuery($query);
		return $db->loadResult();
	}

	/*
	 */
	public static function loadObject($query, $class = \stdClass::class)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');

		$db->setQuery($query);
		return $db->loadObject($class);
	}

	/*
	 */
	public static function loadObjectlist($query, $key=null, $class = \stdClass::class)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');

		$db->setQuery($query);
		return $db->loadObjectlist($key, $class);
	}

	/*
	 */
	public static function loadAssoc($query)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');

		$db->setQuery($query);
		return $db->loadAssoc();
	}

	/*
	 */
	public static function loadAssocList($query, $key=null, $column=null)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');

		$db->setQuery($query);
		return $db->loadAssocList($key, $column);
	}

	/*
	 */
	public static function updateObject($table, &$object, $key, $nulls=false)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');
		return $db->updateObject($table, $object, $key, $nulls);
	}

	/*
	 */
	public static function getQuery($new=true)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');
		return $db->getQuery($new);
	}

	/*
	 */
	public static function execute($sql, $returnId=false, $throwException=true, $showMessage=true)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');

		try
		{
			$db->setQuery($sql);
			if ($db->Execute() && $returnId)
			{
				$result = $db->insertid();
			}
			else
			{
				$result = $db->getAffectedRows();
			}
		}
		catch (\mysqli_sql_exception $e)
		{
			self::save2dmlLog($sql, false, $e);

			if ($throwException)
			{
				throw new \mysqli_sql_exception(ExceptionHelper::getExceptionMessage($e), $e->getCode());
			}

			if ($showMessage)
			{
				Factory::getApplication()->enqueueMessage($e->getCode() . ': ' . $e->getMessage(), 'warning');
			}
			return false;
		}

		self::save2dmlLog($sql, $result);

		return $result;
	}

	/*
	 */
	public static function array2IntegerString($value)
	{
		$result = [];
		foreach((array)$value as $data)
		{
			$result[] = intval($data);
		}
		return implode(',', $result);
	}

	/*
	 */
	protected static function save2dmlLog($sql, $result, $e=null)
	{
		if (class_exists('DmllogHelper'))
		{
			DmllogHelper::save($sql, $result, $e);
		}
	}
}
